#ifndef __AQUAFLY_H__
#define __AQUAFLY_H__

#include <iostream>
#include <cstdio>
#include <cstring>
#include <cstdlib>

#include "ImageSource.h"

#ifdef __cplusplus
extern "C" {
#endif
#include <libraw1394/raw1394.h>
#include <libdc1394/dc1394_control.h>
#include <sys/time.h>
#ifdef __cplusplus
}
#endif

#include "conversions.h"

const int NUMCAMERAS = 2;

// This is now done from the build command line. Makes
// it easier to build without doing and code editing.

// Dragonflies
#define CAMERA_1024x768_8BIT
// #define CAMERA_640x480_8BIT

// BumbleBees
// #define CAMERA_1024x768_16BIT
// #define CAMERA_640x480_16BIT

/* *
 * Define one of the cards below for use.  If
 * your computer has only one 1394 interface
 * card you should be fine with CARD_0.  If
 * not, run testlibraw and see which of your
 * interfaces has your camera on it, then
 * define that one.
 */

/// If we use the PCMCIA FireWire card, we define this on the build command line.
/// Specifies the 1394 interface card to use.

//#ifndef _PCMCIA_
//#define USE_CARD_0
//#else
//#define USE_CARD_1
//#endif
//#define USE_CARD_2
//#define USE_CARD_3

#define USE_CARD_0

class FirewireSource: public ImageSource
{
protected:
    int                   format;
    int                   mode;
    int                   frameRate;
    int                   bytesPerPixel;
    bayer_pattern_t       pattern;
    unsigned long         bufferSize;
    int                   nCard;
    char                  device[25];

    /**
     * Camera independent variables
     */
    raw1394handle_t 	raw1394Handle;
    dc1394_cameracapture dc1394Camera;
    bool			bRaw1394HandleCreated;
    bool			bDc1394CameraCreated;
    bool			isOneShot;
    bool			isStereo;
    bool			isStereoCapable;
    nodeid_t        *cameraNodes;
    int 			activeNode;
    int             numCameras;
    int             numNodes;
    int				nScale, oldScale;

	bool SetupCaptureMode();
	bool GetBayerPattern();
    bool StartStopISO( bool start = true );
    bool DirtyBuffer( int newScale )
    {
    	/// Downsampling changed, hence we have a dirty buffer
    	if( oldScale != newScale )
    	{
    		oldScale = newScale;
    		return true;
    	}
    	return false;
    }
    /// an instance of itself
    static class FirewireSource *_instance;

public:
    FirewireSource();
    ~FirewireSource();
    static class FirewireSource * GetInstance()
	{
		return _instance;
	}
	
	// Getting the image size
	void GetImageSize(int *pXsize, int *pYsize);
	
    /// Removing some stuff from the constructor so that the camera device object does not trigger unless we truly need it.
    void GetDeviceNodes();

    /// Truly initialize the device parameters
    bool InitCamera();
    /// Self explanatory. Stores the frame in the *buffer pointer.
    bool GetFrame();
    /// Generic camera properties method
    void ShowParams() const{};
	/// Utility methods for FireWire devices
	bool IsCameraFree( char *device );
	/// Change downsampling on the fly
	void SetDownSampling( bool isTrue )
	{
		if( isTrue )
			nScale = 2;
		else
			nScale = 1;
	/// Need to resize the frames too
		image_width = dc1394Camera.frame_width / nScale;
		image_height = dc1394Camera.frame_height / nScale;
		std::cout << "Frame dimension now : "<< image_width << "x" << image_height << "\n";
	}

	int ChooseCamera();
	void ToggleOneShot() { isOneShot = !isOneShot; std::cout << "One shot is now " << isOneShot << "\n"; }
	int GetNumberOfCameras() const { return numCameras; }
	bool SwitchNodes();
	bool GetStatus() const { return bDc1394CameraCreated;}
	bool GetStereo() const { return isStereo;}
	bool ToggleStereoMode();
        /// This mode sets up the Region-of-Interest mode for the camera, i.e. the format7 mode.
        bool ROIMode( int tx, int ty, int width, int height );
};

#endif //__AQUAFLY_H__
