/*
 * The following example illustrates how to use C++ code with
 * your C language MEX-file for MATLAB 5 for UNIX or Microsoft Visual.
 * It makes use of member functions, constructors, destructors, and the
 * iostream.
 *
 * The routine simply defines a class, constructs a simple object,
 * and displays the initial values of the internal variables.  It
 * then sets the data members of the object based on the input given
 * to the MEX-file and displays the changed values.
 *
 * The file is called mexcpp.cpp.  The extension was chosen rather
 * arbitrarily and as being unambiguous and generally recognized by
 * C++ compilers.  The mex script should recognize any other common
 * C++ source file extension, such as .C, .cc, or .cxx.
 *
 * FOR UNIX:
 * In order to compile this example, invoke mex as:
 *
 *   mex mexcpp.cpp
 *
 * FOR NT:
 * Be sure to set your environment for MSVC++ 5.0.  In order to do this,
 * run 'mex -setup' at the MATLAB command prompt.  Then invoke mex as:
 *
 *   mex mexcpp.cpp
 *
 *
 * The calling syntax is:    mexcpp( num1, num2 )
 *
 * If you run your MEX-file in MATLAB and you do not receive the expected
 * output make sure that you have a C++ flush() function call in your
 * program.  Also, for NT, cout will not display, since there is no shell
 * to display to.  Therefore, you must use mexPrintf.
 *
 * Copyright 1984-2006 The MathWorks, Inc.
 * All Rights Reserved.
 * $Revision: 1.5.4.3 $
 */
#include <iostream>
#include <math.h>
#include "mex.h"
#include "getframe.h"

using namespace std;

extern void _main();

/****************************/
class MyData {

public:
  void display();
  void set_data(double v1, double v2);
  MyData(double v1 = 0, double v2 = 0);
  ~MyData() { }
private:
  double val1, val2;
};

MyData::MyData(double v1, double v2)
{
  val1 = v1;
  val2 = v2;
}

void MyData::display()
{
#ifdef _WIN32
	mexPrintf("Value1 = %g\n", val1);
	mexPrintf("Value2 = %g\n\n", val2);
#else
  cout << "Value1 = " << val1 << "\n";
  cout << "Value2 = " << val2 << "\n\n";
#endif
}

void MyData::set_data(double v1, double v2) { val1 = v1; val2 = v2; }

/*********************/

static
void mexcpp(
	    double num1,
	    double num2
	    )
{
#ifdef _WIN32
	mexPrintf("\nThe initialized data in object:\n");
#else
  cout << "\nThe initialized data in object:\n";
#endif
  MyData *d = new MyData; // Create a  MyData object
  d->display();           // It should be initialized to
                          // zeros
  d->set_data(num1,num2); // Set data members to incoming
                          // values
#ifdef _WIN32
  mexPrintf("After setting the object's data to your input:\n");
#else
  cout << "After setting the object's data to your input:\n";
#endif
  d->display();           // Make sure the set_data() worked
  delete(d);
  flush(cout);
  return;
}

void mexFunction(
		 int          nlhs,
		 mxArray      *plhs[],
		 int          nrhs,
		 const mxArray *prhs[]
		 )
{
  double      *vin1, *vin2;

  /* Check for proper number of arguments */

  if (nrhs != 0) {
    mexErrMsgTxt("matlab_getframe requires no input argument.");
  } else if (nlhs != 1) {
    mexErrMsgTxt("matlab_getframe requires one output argument.");
  }
  
  unsigned char buffer[256];
  int i,j;
  for (i=0; i<256; i++) {
  	buffer[i] = i;
  }
  
  double *output;
  unsigned char *pImageBuf;
  int Xsize, Ysize, iColor;
  int ImageArraySize;
  int dim[3];

  pImageBuf = GetFrame(&Xsize, &Ysize);
  
  if (pImageBuf == NULL) {
  	  // The camera was not ready, returning 0.0
  	  plhs[0] =mxCreateDoubleMatrix(1,1,mxREAL);
  	  output = mxGetPr(plhs[0]);
  	  (*output) = 0.0;
  	  return;
  }
  ImageArraySize = Xsize*Ysize;

  /* Create the array */
  dim[0] = Ysize;
  dim[1] = Xsize;
  dim[2] = 3; // three color channels.
  
  //cout << "sx:" << dim[0] << "  sy:" << dim[1] << "\n";
  plhs[0] = mxCreateNumericArray(3, dim, mxDOUBLE_CLASS, mxREAL);

  //plhs[0] =mxCreateDoubleMatrix(Xsize,Ysize,mxREAL);
  output = mxGetPr(plhs[0]);

  /* Populate the output */
  //memcpy(output, data, 3*sizeof(double));

  //Alternate method to populate the output, one element at a time 
   for (iColor=0; iColor < 3; iColor++) {
   		for (i = 0; i < Xsize; i++) {
	   		for (j = 0; j < Ysize; j++) {
           		output[i*Ysize + j + ImageArraySize*iColor] 
           		       = 0.00390625*((double)pImageBuf[iColor + 3*i + 3*j*Xsize]); // Scaling by 1/256
       		}
   		}
   }
/*   
   unsigned long PixelIndex = 0;
   for (i = 0; i < (ImageArraySize*3); i+=3) {
       output[PixelIndex] = 0.00390625*((double)pImageBuf[i]); // Scaling by 1/256
       PixelIndex++;
   }
   for (i = 1; i < (ImageArraySize*3); i+=3) {
       output[PixelIndex] = 0.00390625*((double)pImageBuf[i]); // Scaling by 1/256
       PixelIndex++;
   }
   for (i = 2; i < (ImageArraySize*3); i+=3) {
       output[PixelIndex] = 0.00390625*((double)pImageBuf[i]); // Scaling by 1/256
       PixelIndex++;
   } */
  return;
}
